from __future__ import absolute_import

import equibel.PredicateTree.predicate
from equibel.PredicateTree.predicate import *

import equibel.PredicateTree.predicate_parser
from equibel.PredicateTree.predicate_parser import *
