#    Copyright (C) 2014-2015 by
#    Paul Vicol <pvicol@sfu.ca>
#    All rights reserved.
#    MIT license.

from __future__ import absolute_import
from __future__ import print_function

def revise(K, phi):
    """
    """
    pass
